# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rx/proto/rx.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11rx/proto/rx.proto\x12\x02rx\x1a\x1bgoogle/protobuf/empty.proto\"\x95\x01\n\x06Remote\x12\x1f\n\x05image\x18\x01 \x01(\x0b\x32\x10.rx.Remote.Image\x12%\n\x08hardware\x18\x02 \x01(\x0b\x32\x13.rx.Remote.Hardware\x1a$\n\x05Image\x12\x0e\n\x06\x64ocker\x18\x01 \x01(\t\x12\x0b\n\x03tag\x18\x02 \x01(\t\x1a\x1d\n\x08Hardware\x12\x11\n\tprocessor\x18\x01 \x01(\t\"\x08\n\x06Python\"D\n\x0b\x45nvironment\x12\x19\n\x05\x61lloc\x18\x01 \x01(\x0b\x32\n.rx.Remote\x12\x1a\n\x06python\x18\x02 \x01(\x0b\x32\n.rx.Python\"?\n\x05\x44\x65lta\x12\x10\n\x08\x61\x64\x64_path\x18\x01 \x03(\t\x12\x0f\n\x07\x61\x64\x64_dir\x18\x02 \x03(\t\x12\x13\n\x0bremove_path\x18\x03 \x03(\t\"7\n\x06Result\x12\x1c\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x0e.rx.StatusCode\x12\x0f\n\x07message\x18\x02 \x01(\t\"H\n\x0bRsyncSource\x12\x12\n\nmachine_id\x18\x01 \x01(\x03\x12\x11\n\tdirectory\x18\x02 \x01(\t\x12\x12\n\npublic_key\x18\x03 \x01(\x0c\")\n\x10RsyncDestination\x12\x15\n\rdaemon_module\x18\x01 \x01(\t\")\n\x11\x41llocNewWorkerReq\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\"X\n\x0b\x45xecRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x0c\n\x04\x61rgv\x18\x02 \x03(\t\x12%\n\x0crsync_source\x18\x03 \x01(\x0b\x32\x0f.rx.RsyncSource\"v\n\x0c\x45xecResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12\x14\n\x0c\x65xecution_id\x18\x02 \x01(\t\x12\x0e\n\x06stdout\x18\x03 \x01(\x0c\x12\x0e\n\x06stderr\x18\x04 \x01(\x0c\x12\x14\n\x0coutput_files\x18\x05 \x03(\t\"?\n\x0fGetUserResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12\x10\n\x08username\x18\x02 \x01(\t\"o\n\x0bInitRequest\x12\x14\n\x0cproject_name\x18\x01 \x01(\t\x12%\n\x0crsync_source\x18\x02 \x01(\x0b\x32\x0f.rx.RsyncSource\x12#\n\ntarget_env\x18\x03 \x01(\x0b\x32\x0f.rx.Environment\"\x7f\n\x0cInitResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12(\n\nrsync_dest\x18\x02 \x01(\x0b\x32\x14.rx.RsyncDestination\x12\x13\n\x0bworker_addr\x18\x03 \x01(\t\x12\x14\n\x0cworkspace_id\x18\x04 \x01(\t\"*\n\x12InstallDepsRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\"A\n\x13InstallDepsResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result\x12\x0e\n\x06stdout\x18\x02 \x01(\x0c\"9\n\x0bKillRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x14\n\x0c\x65xecution_id\x18\x02 \x01(\t\"&\n\x12SetUsernameRequest\x12\x10\n\x08username\x18\x01 \x01(\t\"1\n\x13SetUsernameResponse\x12\x1a\n\x06result\x18\x01 \x01(\x0b\x32\n.rx.Result*O\n\nStatusCode\x12\x06\n\x02OK\x10\x00\x12\x0b\n\x07UNKNOWN\x10\x01\x12\x0b\n\x07INVALID\x10\x02\x12\r\n\tNOT_FOUND\x10\x03\x12\x10\n\x0cUNAUTHORIZED\x10\x04\x32\xb8\x01\n\x10\x45xecutionService\x12\x42\n\x0bInstallDeps\x12\x16.rx.InstallDepsRequest\x1a\x17.rx.InstallDepsResponse\"\x00\x30\x01\x12-\n\x04\x45xec\x12\x0f.rx.ExecRequest\x1a\x10.rx.ExecResponse\"\x00\x30\x01\x12\x31\n\x04Kill\x12\x0f.rx.KillRequest\x1a\x16.google.protobuf.Empty\"\x00\x32\xf4\x01\n\x0cSetupService\x12;\n\x0e\x41llocNewWorker\x12\x15.rx.AllocNewWorkerReq\x1a\x10.rx.InitResponse\"\x00\x12\x38\n\x07GetUser\x12\x16.google.protobuf.Empty\x1a\x13.rx.GetUserResponse\"\x00\x12+\n\x04Init\x12\x0f.rx.InitRequest\x1a\x10.rx.InitResponse\"\x00\x12@\n\x0bSetUsername\x12\x16.rx.SetUsernameRequest\x1a\x17.rx.SetUsernameResponse\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'rx.proto.rx_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STATUSCODE._serialized_start=1346
  _STATUSCODE._serialized_end=1425
  _REMOTE._serialized_start=55
  _REMOTE._serialized_end=204
  _REMOTE_IMAGE._serialized_start=137
  _REMOTE_IMAGE._serialized_end=173
  _REMOTE_HARDWARE._serialized_start=175
  _REMOTE_HARDWARE._serialized_end=204
  _PYTHON._serialized_start=206
  _PYTHON._serialized_end=214
  _ENVIRONMENT._serialized_start=216
  _ENVIRONMENT._serialized_end=284
  _DELTA._serialized_start=286
  _DELTA._serialized_end=349
  _RESULT._serialized_start=351
  _RESULT._serialized_end=406
  _RSYNCSOURCE._serialized_start=408
  _RSYNCSOURCE._serialized_end=480
  _RSYNCDESTINATION._serialized_start=482
  _RSYNCDESTINATION._serialized_end=523
  _ALLOCNEWWORKERREQ._serialized_start=525
  _ALLOCNEWWORKERREQ._serialized_end=566
  _EXECREQUEST._serialized_start=568
  _EXECREQUEST._serialized_end=656
  _EXECRESPONSE._serialized_start=658
  _EXECRESPONSE._serialized_end=776
  _GETUSERRESPONSE._serialized_start=778
  _GETUSERRESPONSE._serialized_end=841
  _INITREQUEST._serialized_start=843
  _INITREQUEST._serialized_end=954
  _INITRESPONSE._serialized_start=956
  _INITRESPONSE._serialized_end=1083
  _INSTALLDEPSREQUEST._serialized_start=1085
  _INSTALLDEPSREQUEST._serialized_end=1127
  _INSTALLDEPSRESPONSE._serialized_start=1129
  _INSTALLDEPSRESPONSE._serialized_end=1194
  _KILLREQUEST._serialized_start=1196
  _KILLREQUEST._serialized_end=1253
  _SETUSERNAMEREQUEST._serialized_start=1255
  _SETUSERNAMEREQUEST._serialized_end=1293
  _SETUSERNAMERESPONSE._serialized_start=1295
  _SETUSERNAMERESPONSE._serialized_end=1344
  _EXECUTIONSERVICE._serialized_start=1428
  _EXECUTIONSERVICE._serialized_end=1612
  _SETUPSERVICE._serialized_start=1615
  _SETUPSERVICE._serialized_end=1859
# @@protoc_insertion_point(module_scope)
