# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from rx.proto import rx_pb2 as rx_dot_proto_dot_rx__pb2


class ExecutionServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.InstallDeps = channel.unary_stream(
                '/rx.ExecutionService/InstallDeps',
                request_serializer=rx_dot_proto_dot_rx__pb2.InstallDepsRequest.SerializeToString,
                response_deserializer=rx_dot_proto_dot_rx__pb2.InstallDepsResponse.FromString,
                )
        self.Exec = channel.unary_stream(
                '/rx.ExecutionService/Exec',
                request_serializer=rx_dot_proto_dot_rx__pb2.ExecRequest.SerializeToString,
                response_deserializer=rx_dot_proto_dot_rx__pb2.ExecResponse.FromString,
                )
        self.Kill = channel.unary_unary(
                '/rx.ExecutionService/Kill',
                request_serializer=rx_dot_proto_dot_rx__pb2.KillRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )


class ExecutionServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def InstallDeps(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Exec(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Kill(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ExecutionServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'InstallDeps': grpc.unary_stream_rpc_method_handler(
                    servicer.InstallDeps,
                    request_deserializer=rx_dot_proto_dot_rx__pb2.InstallDepsRequest.FromString,
                    response_serializer=rx_dot_proto_dot_rx__pb2.InstallDepsResponse.SerializeToString,
            ),
            'Exec': grpc.unary_stream_rpc_method_handler(
                    servicer.Exec,
                    request_deserializer=rx_dot_proto_dot_rx__pb2.ExecRequest.FromString,
                    response_serializer=rx_dot_proto_dot_rx__pb2.ExecResponse.SerializeToString,
            ),
            'Kill': grpc.unary_unary_rpc_method_handler(
                    servicer.Kill,
                    request_deserializer=rx_dot_proto_dot_rx__pb2.KillRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rx.ExecutionService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ExecutionService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def InstallDeps(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/rx.ExecutionService/InstallDeps',
            rx_dot_proto_dot_rx__pb2.InstallDepsRequest.SerializeToString,
            rx_dot_proto_dot_rx__pb2.InstallDepsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Exec(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/rx.ExecutionService/Exec',
            rx_dot_proto_dot_rx__pb2.ExecRequest.SerializeToString,
            rx_dot_proto_dot_rx__pb2.ExecResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Kill(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rx.ExecutionService/Kill',
            rx_dot_proto_dot_rx__pb2.KillRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class SetupServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AllocNewWorker = channel.unary_unary(
                '/rx.SetupService/AllocNewWorker',
                request_serializer=rx_dot_proto_dot_rx__pb2.AllocNewWorkerReq.SerializeToString,
                response_deserializer=rx_dot_proto_dot_rx__pb2.InitResponse.FromString,
                )
        self.GetUser = channel.unary_unary(
                '/rx.SetupService/GetUser',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=rx_dot_proto_dot_rx__pb2.GetUserResponse.FromString,
                )
        self.Init = channel.unary_unary(
                '/rx.SetupService/Init',
                request_serializer=rx_dot_proto_dot_rx__pb2.InitRequest.SerializeToString,
                response_deserializer=rx_dot_proto_dot_rx__pb2.InitResponse.FromString,
                )
        self.SetUsername = channel.unary_unary(
                '/rx.SetupService/SetUsername',
                request_serializer=rx_dot_proto_dot_rx__pb2.SetUsernameRequest.SerializeToString,
                response_deserializer=rx_dot_proto_dot_rx__pb2.SetUsernameResponse.FromString,
                )


class SetupServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def AllocNewWorker(self, request, context):
        """Creates a new worker for an existing workspace.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUser(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Init(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetUsername(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SetupServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'AllocNewWorker': grpc.unary_unary_rpc_method_handler(
                    servicer.AllocNewWorker,
                    request_deserializer=rx_dot_proto_dot_rx__pb2.AllocNewWorkerReq.FromString,
                    response_serializer=rx_dot_proto_dot_rx__pb2.InitResponse.SerializeToString,
            ),
            'GetUser': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUser,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=rx_dot_proto_dot_rx__pb2.GetUserResponse.SerializeToString,
            ),
            'Init': grpc.unary_unary_rpc_method_handler(
                    servicer.Init,
                    request_deserializer=rx_dot_proto_dot_rx__pb2.InitRequest.FromString,
                    response_serializer=rx_dot_proto_dot_rx__pb2.InitResponse.SerializeToString,
            ),
            'SetUsername': grpc.unary_unary_rpc_method_handler(
                    servicer.SetUsername,
                    request_deserializer=rx_dot_proto_dot_rx__pb2.SetUsernameRequest.FromString,
                    response_serializer=rx_dot_proto_dot_rx__pb2.SetUsernameResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rx.SetupService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class SetupService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def AllocNewWorker(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rx.SetupService/AllocNewWorker',
            rx_dot_proto_dot_rx__pb2.AllocNewWorkerReq.SerializeToString,
            rx_dot_proto_dot_rx__pb2.InitResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUser(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rx.SetupService/GetUser',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            rx_dot_proto_dot_rx__pb2.GetUserResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Init(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rx.SetupService/Init',
            rx_dot_proto_dot_rx__pb2.InitRequest.SerializeToString,
            rx_dot_proto_dot_rx__pb2.InitResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetUsername(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rx.SetupService/SetUsername',
            rx_dot_proto_dot_rx__pb2.SetUsernameRequest.SerializeToString,
            rx_dot_proto_dot_rx__pb2.SetUsernameResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
